#include <mega8.h>
#include <math.h>
#include <delay.h>
#include <stdio.h>



#define F_CPU 8000000
#define REFRESH_CYCLES 5000

#define MAX_A 10   //   A 
#define MAX_B 50   //   B 

#define StartADC()  ADCSRA = ADCSRA | (1<<ADSC) // 

#define EN PORTB.6
#define RS PORTB.0
#define RW PORTB.7

#define SymbolOf(num) num + 48  //     
                               // (,  1  49   ASCII)

unsigned int counter=0; //    
unsigned long sumOfSqr=0; //   
unsigned int adcMAX; //    5000  
unsigned char overflow=0; //   
unsigned char signForMax = ' '; // ,    

unsigned char source = 0;  // 0 -  A (10), 1 -  B(50)
unsigned char prevPC4 = 1;

unsigned long adcValue=0; //   ,    

unsigned char limit; 

//  
void LcdWriteCom(unsigned char data)
{
  RS = 0;
  PORTD = data;   
  EN=1; 
  delay_us(2);
  EN=0;
  delay_us(50);
}
 
//    
void LcdWriteChar(unsigned char data)
{
  RS = 1;     // RS  1
  PORTD = data;        //        
  EN = 1;     //   1
  delay_us(2);
  EN = 0;    //    0
  delay_us(50);
}  

//  
void LcdSetCursor(unsigned char row, unsigned char col)
{
  RS = 0;          //  RS  0
  PORTD = 0x80 + row*64 + col;  //0x80 -       
  EN = 1;    //    1
  delay_us(2);
  EN = 0;    //    0
  delay_us(50);
}

//   float  
void LcdWriteNumber(float value)
{
    unsigned int a;
    if((source == 1) || (value >= 10)) a = (value + 0.05)*100; //  1   ,     50 
    else a = (value + 0.005)*100; //  2 
    
    if(a/1000 == 0 && source ==  1) LcdWriteChar(' '); //    ,    10    1     
    if(a/1000 > 0) //  ,    0
    { 
    LcdWriteChar(SymbolOf(a/1000)); 
    a = a%1000; 
    }
    
    LcdWriteChar(SymbolOf(a/100)); //  
    a = a%100;
    
    LcdWriteChar('.');
    
    LcdWriteChar(SymbolOf(a/10)); //    
    a = a%10;

    if(value < 10 && source == 0) LcdWriteChar(SymbolOf(a)); // 2   ,     
}

//   
void Reset()
    {
        counter=0; 
        sumOfSqr=0;
        adcMAX = 0;
        overflow=0;
        prevPC4 = PINC.4;
    }   

//      
void SelectSource()
    {
        if(source==1) 
        {
            source=0; 
            ADMUX=0xC0; 
            limit = MAX_A;
        } 
        else 
        {  
            source=1; 
            ADMUX=0xC1; 
            limit = MAX_B;
        }
        Reset();
    }


// ADC interrupt service routine
interrupt [ADC_INT] void adc_isr(void)
{
    counter++;
    if(ADCW>512) adcValue=ADCW-512;
    else adcValue=512-ADCW; //  512   
      
    if(adcMAX > 510) {adcValue = 510; overflow = 1;}
    
    if(adcValue > adcMAX) 
    {
        adcMAX = adcValue; //   
        if(ADCW < 512) signForMax = '-';
        else signForMax = ' ';  
    }
    
    
    sumOfSqr = sumOfSqr+adcValue*adcValue; 
    
    StartADC();
}

void main(void)
{

PORTC = 0x00;
DDRC = 0x00;

DDRD = 0xff;
PORTD = 0xff;
   
DDRB = 0xff;

RS = 1;
EN = 1;
RW = 0;


// 
delay_ms(40);
LcdWriteCom(0x38);
LcdWriteCom(0x0c);
LcdWriteCom(0x01);
delay_ms(2);
LcdWriteCom(0x06);  





// ADC initialization
// ADC Clock frequency: 125,000 kHz
// ADC Voltage Reference: Int., cap. on AREF
ADMUX=0xC1;
ADCSRA=0x8E;
SelectSource();

// Global enable interrupts
#asm("sei")


StartADC();

  LcdSetCursor(0, 0);
  LcdWriteChar('R');
  LcdWriteChar('M');
  LcdWriteChar('S');
  LcdWriteChar(' ');
  
  LcdSetCursor(1, 0);
  LcdWriteChar('M');
  LcdWriteChar('A');
  LcdWriteChar('X');
  LcdWriteChar(' ');


while (1)
{  
      if(PINC.4==0 && prevPC4 == 1) 
      { 
        delay_ms(10);
        if(PINC.4==0) SelectSource(); //   
      }  
      
      if(counter >= REFRESH_CYCLES)
      {   
        LcdSetCursor(1, 3); LcdWriteChar(' '); //  '>',   
        LcdSetCursor(0, 3); LcdWriteChar(' ');
        
        LcdSetCursor(0, 4);  //   1-   5- 
        LcdWriteNumber(limit*(sqrt(sumOfSqr/counter))/510);// RMS 
        LcdSetCursor(1, 4); //   2-   5- 
        LcdWriteNumber(limit*(float)adcMAX/510);//  
        
        if(source == 1 && limit*(float)adcMAX/510 < 10) LcdSetCursor(1, 4);
        else LcdSetCursor(1, 3);
        LcdWriteChar(signForMax);  //  
        
        
        if(overflow) 
        {
        LcdSetCursor(0, 3); LcdWriteChar('>');
        LcdSetCursor(1, 3); LcdWriteChar('>');
        } 
            
         Reset();
      }
} 
}